unit Main;
{$H+,B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs,
  Menus, XComDrv, ToolWin, ComCtrls, ExtCtrls, Buttons, StdCtrls, Graphics;

type

  TXMemo = class(TMemo)
  protected
    procedure WndProc(var Message: TMessage); override;
  end;

  TFMain = class(TForm)
    Modem              : TXModem;  
    Menu               : TMainMenu;
    SBar               : TStatusBar;    
    N5                 : TMenuItem;
    ECS1               : TMenuItem;
    MIDevice           : TMenuItem;
    MIOpen             : TMenuItem;
    MIClose            : TMenuItem;
    N1                 : TMenuItem;
    MIMonitorEvents    : TMenuItem;
    MISettings         : TMenuItem;
    N2                 : TMenuItem;
    MIExit             : TMenuItem;
    MIModem            : TMenuItem;
    MIOptions          : TMenuItem;
    N3                 : TMenuItem;
    MIInitialize       : TMenuItem;
    MIReset            : TMenuItem;
    N4                 : TMenuItem;
    MIConnect          : TMenuItem;
    MIDisconnect       : TMenuItem;
    CTime              : TTimer;
    CB: TControlBar;
    P1: TPanel;
    DOpen: TSpeedButton;
    DClose: TSpeedButton;
    S1: TBevel;
    DSet: TSpeedButton;
    S2: TBevel;
    DCState: TSpeedButton;
    P2: TPanel;
    Edit: TEdit;
    DSend: TButton;
    P3: TPanel;
    MOpt: TSpeedButton;
    S3: TBevel;
    MInit: TSpeedButton;
    MReset: TSpeedButton;
    S4: TBevel;
    MCon: TSpeedButton;
    MDis: TSpeedButton;
    procedure DCloseClick(Sender: TObject);
    procedure DOpenClick(Sender: TObject);
    procedure MIExitClick(Sender: TObject);
    procedure DSetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DSendClick(Sender: TObject);
    procedure MInitClick(Sender: TObject);
    procedure MResetClick(Sender: TObject);
    procedure ModemHayesAT(Sender: TObject; AT: THayesAT);
    procedure MOptClick(Sender: TObject);
    procedure MConClick(Sender: TObject);
    procedure MDisClick(Sender: TObject);
    procedure ModemDisconnect(Sender: TObject);
    procedure ModemRing(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ModemConnecting(Sender: TObject;
      const ConnectType: TConnectType);
    procedure ModemDisconnecting(Sender: TObject);
    procedure ModemConnect(Sender: TObject);
    procedure MIModemClick(Sender: TObject);
    procedure DCStateClick(Sender: TObject);
    procedure ECS1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ModemRead(Sender: TObject);
    procedure ModemSend(Sender: TObject);
    procedure ModemChangeCmdState(Sender: TObject);
    procedure ModemCommEvent(Sender: TObject; const Events: TDeviceEvents);
    procedure ModemData(Sender: TObject; const Received: Cardinal);
    procedure MIMonitorEventsClick(Sender: TObject);
    procedure CTimeTimer(Sender: TObject);
    procedure MIDeviceClick(Sender: TObject);
  private
    { Private declarations }
    Memo: TXMemo;
    InConnecting: Boolean;
    function Panel1Text: string;
  public
  end;

var
  FMain: TFMain;

implementation

uses DevSet, ModSet, EvFrm;

{$R *.DFM}

procedure TXMemo.WndProc(var Message: TMessage);
begin
  inherited WndProc(Message);
  if (Message.Msg = WM_SETFOCUS) then HideCaret(Handle);
end;

var
  BufStr: string='';
  Ti: DWORD;

function TimeCon: string;
var sec, min: DWORD;
  function ToStr(m: dword): string;
  begin
    Result := IntToStr(m);
    if m<10 then Result := '0'+Result
  end;
begin
  sec := (GetTickCount-Ti) div 1000;
  min := sec div 60;
  min := min mod 60;
  sec := sec mod 60;
  Result := ToStr(min)+':'+ToStr(sec);
end;  
  
function TFMain.Panel1Text: string;
begin
  if Modem.Installed then
    Result := 'Modem detected'
  else
    Result := 'Device opened';  
end;

procedure TFMain.DCloseClick(Sender: TObject);
begin
  if not Modem.Opened then Exit;
  if (msConnected in Modem.ModemState) then
  begin
    Memo.Lines.Add('Closing while connected.');
    Modem.LowerDTR;
  end;
  Modem.CloseDevice;
  SBar.Panels[0].Text := 'Idle...';
  SBar.Panels[1].Text := '';
  SBar.Panels[2].Text := '';
  MIClose.Enabled := False;
  DClose.Enabled := False;
  MIOpen.Enabled := True;
  DOpen.Enabled := True;
  MOpt.Enabled := True;
  MInit.Enabled := False;
  MReset.Enabled := False;
  MCon.Enabled := False;
  MDis.Enabled := False;
  Edit.Enabled := False;
  DSend.Enabled := False;
  MIOptions.Enabled := DOpen.Enabled;
  MIInitialize.Enabled := MInit.Enabled;
  MIReset.Enabled := MReset.Enabled;
  MIConnect.Enabled := MCon.Enabled;
  MIDisconnect.Enabled := MDis.Enabled;
  DCState.Enabled := False;
  DCState.Hint := '';
  Memo.Lines.Add('Device closed.');
end;

procedure TFMain.DOpenClick(Sender: TObject);
begin
  if Modem.Opened then Exit;
  FEvents.List.Clear;
  with Memo, Lines do
  begin
    Clear;
    Repaint;
    Add('Opening device.');
    Add('Profile:');
    Add('  Port >          '+FDev.eName.Text);
    Add('  Baud rate >     '+FDev.eBaud.Text);
    Add('  Flow control >  '+FDev.cbFlow.Text);
    Add('  Data bits >     '+FDev.cbData.Text);
    Add('  Parity >        '+FDev.cbParity.Text);
    Add('  Stop bits >     '+FDev.cbStop.Text);
  end;

  SBar.Panels[1].Text := 'Received: 0';
  SBar.Panels[2].Text := 'Sent: 0';
  try
    MIClose.Enabled := Modem.OpenDevice;
    if Modem.Opened
      then Memo.Lines.Add('Device succesfully opened.')
      else Memo.Lines.Add('Failed to open device.');
  except
    on E: Exception do
      Memo.Lines.Add('Failed to open device. '+E.Message);
  end;

  if Modem.Opened
    then SBar.Panels[0].Text := Panel1Text
    else Exit;

  DClose.Enabled := True;
  MIOpen.Enabled := False;
  DOpen.Enabled := False;

  Edit.Enabled := True;
  DSend.Enabled := True;
  DCState.Down := False;
  DCState.Enabled := Modem.Installed;
  DCState.Hint := 'Enter command state';

end;

procedure TFMain.MIExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFMain.DSetClick(Sender: TObject);
begin
  FDev.ShowModal;
end;

procedure TFMain.FormCreate(Sender: TObject);
begin
  InConnecting := False;
  Modem.BaudRate := brCustom;
  Memo := TXMemo.Create(Self);
  Memo.Parent := Self;
  Memo.WordWrap := False;
  Memo.ScrollBars := ssBoth;
  Memo.Align := alClient;
  Memo.ReadOnly := True;
  Memo.TabStop := False;
  Memo.Cursor := crArrow;
end;

procedure TFMain.DSendClick(Sender: TObject);
var Async: THandle;
begin
  if Edit.Enabled and not Edit.Focused then Edit.SetFocus;
  if (msCommandState in Modem.ModemState) then
    Modem.SendCommand(Edit.Text)
  else if not (loSend in Modem.Locked) then
  begin
    Modem.PurgeIn;
    BufStr := '';
    if (Memo.Lines[Memo.Lines.Count-1]<>'') then Memo.Lines.Add('');
    Async := Modem.InitAsync(nil, True);
    Modem.SendStringAsync(Async, Edit.Text+#13#10);
  end;
end;

procedure TFMain.MInitClick(Sender: TObject);
begin
  with Memo.Lines, Modem, ModemSettings do
  begin
    Add('Initializing modem.');
    Add(InitString);
    if InitModem
      then Add('Passed.')
      else Add('Failed.');
  end;
end;

procedure TFMain.MResetClick(Sender: TObject);
begin
  with Memo.Lines, Modem, ModemSettings do
  begin
    Add('Resetting modem.');
    Add(ResetString);
    if ResetModem
      then Add('Passed.')
      else Add('Failed.');
  end;
end;

procedure TFMain.ModemHayesAT(Sender: TObject; AT: THayesAT);
begin
  if InConnecting then InConnecting := False;
  Memo.Lines.Add('['+TimeToStr(Time)+'] - AT received'#13#10+AT.Command);
  if (AT.Data<>'') then Memo.Lines.Add(AT.Data);
  Memo.Lines.Add(AT.Result+#13#10);
end;

procedure TFMain.MOptClick(Sender: TObject);
begin
  FMod.ShowModal;
end;

procedure TFMain.MConClick(Sender: TObject);
begin
  Modem.Connect(False);
end;

procedure TFMain.MDisClick(Sender: TObject);
begin
  if not Modem.Opened then Exit;
  SBar.Panels[0].Text := 'Disconnecting...';
  if Modem.Connected and (coLeaveDTROpen in Modem.Options)
  and not (msConnecting in Modem.ModemState)
    then Modem.LowerDTR
    else Modem.Disconnect;
  SBar.Panels[0].Text := Panel1Text;
  MOpt.Enabled := True;
  MInit.Enabled := True;
  MReset.Enabled := True;
  MCon.Enabled := True;
  MDis.Enabled := False;
  MIInitialize.Enabled := MInit.Enabled;
  MIReset.Enabled := MReset.Enabled;
  MIConnect.Enabled := MCon.Enabled;
  MIOptions.Enabled := MOpt.Enabled;
  MIDisconnect.Enabled := MDis.Enabled;
end;

procedure TFMain.ModemDisconnect(Sender: TObject);
begin
  CTime.Enabled := False;
  Memo.Lines.Add('Disconnected. Connection - '+TimeCon);
  SBar.Panels[0].Text := Panel1Text;
end;

procedure TFMain.ModemRing(Sender: TObject);
begin
  Memo.Lines.Add('RING detected.');
end;

procedure TFMain.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then DSend.Click;
end;

procedure TFMain.ModemConnecting(Sender: TObject;
  const ConnectType: TConnectType);
begin
  InConnecting := True;
  with Memo.Lines do
  begin
    Add('Connecting.');
    case ConnectType of
      ctDial:   SBar.Panels[0].Text := 'Dialing...';
      ctWait:   SBar.Panels[0].Text := 'Waiting for call...';
      ctDirect: SBar.Panels[0].Text := 'Direct connection...';
    end;

    Add(SBar.Panels[0].Text);
    MInit.Enabled := False;
    MReset.Enabled := False;
    MCon.Enabled := False;
    MDis.Enabled := True;
    DCState.Enabled := False;
    DCState.Hint := '';
    MIInitialize.Enabled := MInit.Enabled;
    MIReset.Enabled := MReset.Enabled;
    MIConnect.Enabled := MCon.Enabled;
    MIDisconnect.Enabled := True;
  end;
end;

procedure TFMain.ModemDisconnecting(Sender: TObject);
begin
  if InConnecting then
    Memo.Lines.Add('Unable to connect.');
  SBar.Panels[0].Text := Panel1Text;
  MOpt.Enabled := True;
  MInit.Enabled := True;
  MReset.Enabled := True;
  MCon.Enabled := True;
  MDis.Enabled := False;
  MIInitialize.Enabled := MInit.Enabled;
  MIReset.Enabled := MReset.Enabled;
  MIConnect.Enabled := MCon.Enabled;
  MIOptions.Enabled := MOpt.Enabled;
  MIDisconnect.Enabled := MDis.Enabled;
  DCState.Hint := 'Exit command state';
  DCState.Down := True;
  DCState.Enabled := True;
end;

procedure TFMain.ModemConnect(Sender: TObject);
begin
  InConnecting := False;
  DCState.Enabled := True;
  DCState.Down := False;
  Ti := GetTickCount;
  CTime.Enabled := True;
  DCState.Hint := 'Enter command state';
  SBar.Panels[0].Text := 'Connected';
  Memo.Lines.Add(SBar.Panels[0].Text);
end;

procedure TFMain.MIModemClick(Sender: TObject);
begin
  if Modem.Opened and Modem.Installed then
  begin
    ECS1.Visible := True;
    if msCommandState in Modem.ModemState
      then ECS1.Caption := '&Exit command state'
      else ECS1.Caption := '&Enter command state';
  end else ECS1.Visible := False;
  N5.Visible := ECS1.Visible;
end;

procedure TFMain.DCStateClick(Sender: TObject);
var Update: Boolean;
begin
  if (Modem.BaudValue>=MB_MINBAUD) then
  begin
    if (msCommandState in Modem.ModemState)
      then Update := not Modem.ExitCommandState
      else Update := not Modem.EnterCommandState;
  end else Update := True;
  if Update then
    DCState.Down := not DCState.Down;
end;

procedure TFMain.ECS1Click(Sender: TObject);
begin
  DCState.Click;
  DCState.Down := not DCState.Down;
end;

procedure TFMain.FormDestroy(Sender: TObject);
begin
  Memo.Free;
end;

const MW = 80;

procedure TFMain.ModemRead(Sender: TObject);
var
  s: string;
  I: Integer;
begin
  s := Format('Received: %d', [Modem.TotalReceived]);
  I := SBar.Canvas.TextWidth(s)+6;
  if I<MW then I := MW;
  SBar.Panels[1].Width := I;
  SBar.Panels[1].Text := s;
end;

procedure TFMain.ModemSend(Sender: TObject);
var
  s: string;
  I: Integer;
begin
  s := Format('Sent: %d', [Modem.TotalSent]);
  I := SBar.Canvas.TextWidth(s)+6;
  if I<MW then I := MW;
  SBar.Panels[2].Width := I;
  SBar.Panels[2].Text := s;
end;

procedure TFMain.ModemChangeCmdState(Sender: TObject);
begin
  if (msCommandState in Modem.ModemState) then
  begin
    DCState.Hint := 'Exit command state';
    Memo.Lines.Add('Entered into command state');
    DCState.Down := True;
  end else
  begin
    DCState.Hint := 'Enter command state';
    Memo.Lines.Add('Command state not present');
    DCState.Down := False;    
  end;
  if (msCommandState in Modem.ModemState) then
  begin
    MInit.Enabled := not Modem.Connected;
    MReset.Enabled := not Modem.Connected;
    MCon.Enabled := not Modem.Connected;
    MDis.Enabled := Modem.Connected;
  end else
  begin
    MInit.Enabled := False;
    MReset.Enabled := False;
    MCon.Enabled := False;
    MDis.Enabled := False;
  end;
  MIInitialize.Enabled := MInit.Enabled;
  MIReset.Enabled := MReset.Enabled;
  MIConnect.Enabled := MCon.Enabled;
  MIDisconnect.Enabled := MDis.Enabled;
end;

type tcs= set of char;

procedure RemoveChars( const ch: tcs; var s: string );
var i: integer;
begin
  i := 1;
  while (i<=Length(s)) do
    if s[i] in ch then
      Delete(s, i, 1)
    else
      Inc(i);
end;

function PBreak(var s: string): Integer;
begin
  for Result:=1 to Length(s) do
    if s[Result]<#32 then Exit;
  Result := 0;
end;

function GetLine(var s, s1: string): Boolean;
var I: Integer;
begin
  I := PBreak(s);
  if (I>0) then
  begin
    s1 := Copy(s, 1, I-1);
    Delete(s, 1, I);
    Result := True;
    RemoveChars([#0..#31], s1);
  end else Result := False;
end;

procedure TFMain.ModemCommEvent(Sender: TObject;
  const Events: TDeviceEvents);
begin
  if deDSR in Events then
  begin
    DCState.Enabled := Modem.Installed;
    DCState.Down := msCommandState in Modem.ModemState;
    SBar.Panels[0].Text := Panel1Text;
  end;
  if (Events<>[]) then
    FEvents.AddEvents(Events);
end;

procedure TFMain.ModemData(Sender: TObject; const Received: Cardinal);
var s: string;
  procedure TrimChars;
  var I: Integer;
  begin
    I := 1;
    while I<=Length(s) do
      if (s[I] = #10) then
        Delete(s, I, 1)
      else
        Inc(I);
    I := 1;        
    while I<=Length(s) do
    begin
      if (s[I] = #13) then
      begin
        Insert(#10, s, I+1);
        Inc(I);
      end;
      Inc(I);
    end;
  end;
begin
  if not (msCommandState in Modem.ModemState) and not (loRead in Modem.Locked) and Modem.ReadString(s) then
  begin
    Memo.SelStart := 65535;
    Memo.SelLength := 0;
    Memo.SelText := s;
  end;
end;

procedure TFMain.MIMonitorEventsClick(Sender: TObject);
begin
  MIMonitorEvents.Checked := not MIMonitorEvents.Checked;
  FEvents.Visible := MIMonitorEvents.Checked;
end;

procedure TFMain.CTimeTimer(Sender: TObject);
begin
  SBar.Panels[0].Text := 'Connected '+TimeCon;
end;

procedure TFMain.MIDeviceClick(Sender: TObject);
begin
  MIMonitorEvents.Checked := FEvents.Visible;
end;

end.
 